@@ -1,4 +1,5 @@
-auth_token <- NA
+Quandl.auth_token <- NA
+Quandl.remaining_limit <- NA
 
 #' Query or set Quandl API token
 #' @param auth_token Optionally passed parameter to set Quandl \code{auth_token}.
@@ -11,9 +12,37 @@ auth_token <- NA
 Quandl.auth <- function(auth_token) {
 
     if (!missing(auth_token))
-        assignInNamespace('auth_token', auth_token, 'Quandl')
+        assignInNamespace('Quandl.auth_token', auth_token, 'Quandl')
 
-    invisible(Quandl:::auth_token)
+    invisible(Quandl.auth_token)
+
+}
+
+#' Query remaining API limit
+#' @param remaining_limit Optionally passed parameter to update Quandl \code{remaining_limit}
+#' @param force_check Forces the function to requery Quandl for the api limit remaining, could be used after an authentication token change.
+#' @return Returns the number of remaining API calls.
+#' @seealso \code{\link{Quandl}}
+#' @examples \dontrun{
+#' Quandl.limit(700)
+#' }
+#' @importFrom RCurl getURL
+#' @importFrom RCurl basicHeaderGatherer
+#' @export
+Quandl.limit <- function(remaining_limit, force_check=FALSE) {
+    if (!missing(remaining_limit)) {
+        assignInNamespace('Quandl.remaining_limit', remaining_limit, 'Quandl')
+    }
+    else if (is.na(Quandl.remaining_limit) || force_check) {
+        headers <- basicHeaderGatherer()
+        if (is.na(Quandl.auth()))
+            getURL("http://www.quandl.com/api/v1/datasets/TAMMER/RANDOM.json?exclude_data=true", headerfunction = headers$update)
+        else
+            getURL(paste("http://www.quandl.com/api/v1/datasets/TAMMER/RANDOM.json?auth_token=", Quandl.auth(), "&exclude_data=true", sep=""), headerfunction = headers$update)
+        assignInNamespace('Quandl.remaining_limit', headers$value()[["X-RateLimit-Remaining"]], 'Quandl')
+    }
+
+    return(Quandl.remaining_limit)
 
 }
 
@@ -22,13 +51,14 @@ Quandl.auth <- function(auth_token) {
 #' An authentication token is needed for access to the Quandl API multiple times. Set your \code{access_token} with \code{Quandl.auth} function.
 #'
 #' For instructions on finding your authentication token go to www.quandl.com/API
-#' @param code Dataset code on Quandl specified as a string.
+#' @param code Dataset code on Quandl specified as a string or an array of strings.
 #' @param type Type of data returned specified as string. Can be 'raw', 'ts', 'zoo' or 'xts'.
 #' @param start_date Use to truncate data by start date in 'yyyy-mm-dd' format.
 #' @param end_date Use to truncate data by end date in 'yyyy-mm-dd' format.
 #' @param transformation Apply Quandl API data transformations.
 #' @param collapse Collapse frequency of Data.
 #' @param rows Select number of dates returned.
+#' @param sort Select if data is given to R in ascending or descending formats. Helpful for the rows parameter.
 #' @param meta Returns meta data in list format as well as data.
 #' @param authcode Authentication Token for extended API access by default set by \code{\link{Quandl.auth}}.
 #' @return Depending on the outpug flag the class is either data.frame, time series, xts, zoo or a list containing one.
@@ -39,18 +69,24 @@ Quandl.auth <- function(auth_token) {
 #' quandldata = Quandl("NSE/OIL", collapse="monthly", start_date="2013-01-01", type="ts")
 #' plot(quandldata[,1])
 #' }
+#' @importFrom RCurl getURL
+#' @importFrom RCurl basicHeaderGatherer
 #' @importFrom RJSONIO fromJSON
 #' @importFrom zoo zoo
 #' @importFrom xts xts
 #' @export
-Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_date, transformation = c('', 'diff', 'rdiff', 'normalize', 'cumul'), collapse = c('', 'weekly', 'monthly', 'quarterly', 'annual'), rows, meta = FALSE, authcode = Quandl.auth()) {
+Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_date, transformation = c('', 'diff', 'rdiff', 'normalize', 'cumul', 'rdiff_from'), collapse = c('', 'weekly', 'monthly', 'quarterly', 'annual'), rows, sort = c('desc', 'asc'), meta = FALSE, authcode = Quandl.auth()) {
 
     ## Flag to indicate frequency change due to collapse
     freqflag = FALSE
+    ## Default to single dataset
+    multiset = FALSE
     ## Check params
     type           <- match.arg(type)
     transformation <- match.arg(transformation)
     collapse       <- match.arg(collapse)
+    sort           <- match.arg(sort)
+
 
     ## Helper function
     frequency2integer <- function(freq) {
@@ -62,18 +98,34 @@ Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_da
                1)
     }
 
+    if (!all(gsub("[^A-Z0-9_./]", "", code) == code))
+        stop("Codes are comprised of capital letters, numbers and underscores only.")
     ## Build API URL and add auth_token if available
-    string <- paste("http://www.quandl.com/api/v1/datasets/", code, ".json?sort_order=asc&", sep="")
+    if (length(code) == 1)
+        string <- paste("http://www.quandl.com/api/v1/datasets/", code, ".json?", sep="")
+    else {
+        multiset = TRUE
+        freqflag = TRUE ## Frequency not automatically supported with multisets
+        freq <- 1
+        string <- "http://www.quandl.com/api/v1/multisets.json?columns="
+        string <- paste(string, sub("/",".",code[1]), sep="")
+        for (i in 2:length(code)) {
+            string <- paste(string, sub("/",".",code[i]), sep=",")
+        }
+    }
     if (is.na(authcode))
         warning("It would appear you aren't using an authentication token. Please visit http://www.quandl.com/help/r or your usage may be limited.")
     else
         string <- paste(string, "&auth_token=", authcode, sep = "")
 
+
     ## Add API options
     if (!missing(start_date))
         string <- paste(string, "&trim_start=", as.Date(start_date), sep = "")
     if (!missing(end_date))
         string <- paste(string,"&trim_end=", as.Date(end_date) ,sep = "")
+    if (sort %in% c("asc", "desc"))
+        string <- paste(string, "&sort_order=", sort, sep = "")    
     if (transformation %in% c("diff", "rdiff", "normalize", "cumul"))
         string <- paste(string,"&transformation=", transformation, sep = "")
     if (collapse %in% c("weekly", "monthly", "quarterly", "annual")) {
@@ -82,17 +134,31 @@ Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_da
         freqflag = TRUE
     }
     if (!missing(rows))
-        string <- paste(string,"&rows=", rows ,sep = "")
+        string <- paste(string,"&limit=", rows ,sep = "")
+
 
     ## Download and parse data
-    json <- try(fromJSON(string, nullValue = as.numeric(NA)), silent = TRUE)
+    headers <- basicHeaderGatherer()
+    response <- getURL(string, headerfunction = headers$update)
+    Quandl.limit(headers$value()[["X-RateLimit-Remaining"]])
 
-    ## Check if code exists
-    if (inherits(json, 'try-error'))
-        stop("Code does not exist")
-    if (length(json) == 0)
-        stop("Code does not exist")
+    if (length(grep("403", headers$value()[["status"]]))) {
+        stop(response)
+    }
+    json <- try(fromJSON(response, nullValue = as.numeric(NA)), silent = TRUE)
 
+
+    ## Error Catching
+    if (inherits(json, 'try-error')) {
+        error_str <- paste("Something is wrong, and you got past my error catching. Please copy this output and email to connect@quandl.com:", string, sep="\n")
+        stop(error_str)
+    }
+    if (json["error"] == "Requested entity does not exist." || json["error"] == "Unknown api route.")
+        stop("Code does not exist")
+    if (length(json$data) == 0)
+        stop("Requested Entity does not exist.")
+    if (length(json$column_names) > 100 && multiset)
+        stop("Currently we only support multisets with up to 100 columns. Please contact connect@quandl.com if this is a problem.")
     ## Detect frequency
     if (!freqflag)
         freq <- frequency2integer(json$frequency)
@@ -139,20 +205,19 @@ Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_da
     ## Returning xts object
     if (type == "xts")
         data_out <- xts(data[c(-1)],data[,1])
-
     ## Append Metadata
-    if (meta) {
+    if (meta && !multiset) {
         output <- list()
         output$data <- data_out
         output$frequency <-json$frequency
         output$name <- json$name
         output$description <- json$description
         output$updated <- json$updated_at
-        source_code <- strsplit(code, "/")
-        source_code <- source_code[[1]][1]
-        source_string <- paste("http://www.quandl.com/api/v1/sources/", source_code, ".json", sep="")
+        output$source_code <- json$source_code
+        output$code <- paste(output$source_code, json$code, sep="/")
+        source_string <- paste("http://www.quandl.com/api/v1/sources/", output$source_code, ".json", sep="")
         if (!is.na(authcode))
-            source_string <- paste(source_string, "&auth_token=", authcode, sep = "")
+            source_string <- paste(source_string, "?auth_token=", authcode, sep = "")
         source_json <- fromJSON(source_string, nullValue = as.numeric(NA))
         output$source_name <- source_json$name
         output$source_link <- source_json$host
@@ -162,7 +227,4 @@ Quandl <- function(code, type = c('raw', 'ts', 'zoo', 'xts'), start_date, end_da
     else {
         return(data_out)
     }
-    ## Just in case
-    stop("Invalid Type")
-
 }
